@@ -1,6 +1,9 @@
 #![no_std]
 #![forbid(missing_docs)]
 
+// The safety requirement is "use the procedural derive".
+#![allow(clippy::missing_safety_doc)]
+
 //! A library for defining enums that can be used in compact bit sets. It supports enums up to 128
 //! variants, and has a macro to use these sets in constants.
 //!
@@ -18,7 +21,8 @@
 //! }
 //! ```
 //!
-//! For more information on more advanced use cases, see the documentation for [`EnumSetType`].
+//! For more information on more advanced use cases, see the documentation for
+//! [`#[derive(EnumSetType)]`](./derive.EnumSetType.html).
 //!
 //! # Working with EnumSets
 //!
@@ -75,8 +79,6 @@
 //! assert_eq!(set, Enum::A | Enum::E | Enum::G);
 //! ```
 
-pub use enumset_derive::EnumSetType;
-
 use core::cmp::Ordering;
 use core::fmt;
 use core::fmt::{Debug, Formatter};
@@ -127,30 +129,54 @@ use crate::__internal::EnumSetTypePrivate;
 mod repr;
 use crate::repr::EnumSetTypeRepr;
 
-/// The trait used to define enum types that may be used with [`EnumSet`].
+/// The procedural macro used to derive [`EnumSetType`], and allow enums to be used with
+/// [`EnumSet`].
 ///
-/// This trait should be implemented using `#[derive(EnumSetType)]`. Its internal structure is
-/// not stable, and may change at any time.
+/// It may be used with any enum with no data fields, at most 127 variants, and no variant
+/// discriminators larger than 127.
 ///
-/// # Custom Derive
+/// # Additional Impls
 ///
-/// Any C-like enum is supported, as long as there are no more than 128 variants in the enum,
-/// and no variant discriminator is larger than 127.
+/// In addition to the implementation of `EnumSetType`, this procedural macro creates multiple
+/// other impls that are either required for the macro to work, or make the procedural macro more
+/// ergonomic to use.
 ///
-/// The custom derive for [`EnumSetType`] automatically creates implementations of [`PartialEq`],
-/// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] allowing the enum to be used as
-/// if it were an [`EnumSet`] in expressions. This can be disabled by adding an `#[enumset(no_ops)]`
-/// annotation to the enum.
+/// A full list of traits implemented as is follows:
 ///
-/// The custom derive for `EnumSetType` automatically implements [`Copy`], [`Clone`], [`Eq`], and
-/// [`PartialEq`] on the enum. These are required for the [`EnumSet`] to function.
+/// * [`Copy`], [`Clone`], [`Eq`], [`PartialEq`] implementations are created to allow `EnumSet`
+///   to function properly. These automatic implementations may be suppressed using
+///   `#[enumset(no_super_impls)]`, but these traits must still be implemented in another way.
+/// * [`PartialEq`], [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] implementations are
+///   created to allow the crate to be used more ergonomically in expressions. These automatic
+///   implementations may be suppressed using `#[enumset(no_ops)]`.
 ///
-/// In addition, if you have renamed the `enumset` crate in your crate, you can use the
-/// `#[enumset(crate_name = "enumset2")]` attribute to tell the custom derive to use that name
-/// instead.
+/// # Options
 ///
-/// Attributes controlling the serialization of an `EnumSet` are documented in
-/// [its documentation](./struct.EnumSet.html#serialization).
+/// Options are given with `#[enumset(foo)]` annotations attached to the same enum as the derive.
+/// Multiple options may be given in the same annotation using the `#[enumset(foo, bar)]` syntax.
+///
+/// A full list of options is as follows:
+///
+/// * `#[enumset(no_super_impls)]` prevents the derive from creating implementations required for
+///   [`EnumSet`] to function. When this attribute is specified, implementations of [`Copy`],
+///   [`Clone`], [`Eq`], and [`PartialEq`]. This can be useful if you are using a code generator
+///   that already derives these traits. These impls should function identically to the
+///   automatically derived versions, or unintentional behavior may be a result.
+/// * `#[enumset(no_ops)` prevents the derive from implementing any operator traits.
+/// * `#[enumset(crate_name = "enumset2")]` may be used to change the name of the `enumset` crate
+///   used in the generated code. By default enumset parses `Cargo.toml` to determine the name of
+///   the crate, or falls back to `enumset` if it cannot be parsed.
+///
+/// When the `serde` feature is used, the following features may also be specified. These options
+/// may be used (with no effect) when building without the feature enabled:
+///
+/// * `#[enumset(serialize_repr = "u8")]` may be used to specify the integer type used to serialize
+///   the underlying bitset.
+/// * `#[enumset(serialize_as_list)]` may be used to serialize the bitset as a list of enum
+///   variants instead of an integer. This requires [`Deserialize`] and [`Serialize`] be
+///   implemented on the enum.
+/// * `#[enumset(serialize_deny_unknown)]` causes the generated deserializer to return an error
+///   for unknown bits instead of silently ignoring them.
 ///
 /// # Examples
 ///
@@ -184,14 +210,36 @@ use crate::repr::EnumSetTypeRepr;
 ///    A, B, C, D, E, F, G,
 /// }
 /// ```
+pub use enumset_derive::EnumSetType;
+
+/// The trait used to define enum types that may be used with [`EnumSet`].
+///
+/// This trait must be impelmented using `#[derive(EnumSetType)]`, is not public API, and its
+/// internal structure may change at any time with no warning.
+///
+/// For full documentation on the procedural derive and its options, see
+/// [`#[derive(EnumSetType)]`](./derive.EnumSetType.html).
 pub unsafe trait EnumSetType: Copy + Eq + EnumSetTypePrivate { }
 
 /// An efficient set type for enums.
 ///
 /// It is implemented using a bitset stored using the smallest integer that can fit all bits
-/// in the underlying enum. In general, an enum variant with a numeric value of `n` is stored in
+/// in the underlying enum. In general, an enum variant with a discriminator of `n` is stored in
 /// the nth least significant bit (corresponding with a mask of, e.g. `1 << enum as u32`).
 ///
+/// # Numeric representation
+///
+/// `EnumSet` is internally implemented using integer types, and as such can be easily converted
+/// from and to numbers.
+///
+/// Each bit of the underlying integer corresponds to at most one particular enum variant. If the
+/// corresponding bit for a variant is set, it present in the set. Bits that do not correspond to
+/// any variant are always unset.
+///
+/// By default, each enum variant is stored in a bit corresponding to its discriminator. An enum
+/// variant with a discriminator of `n` is stored in the `n + 1`th least significant bit
+/// (corresponding to a mask of e.g. `1 << enum as u32`).
+///
 /// # Serialization
 ///
 /// When the `serde` feature is enabled, `EnumSet`s can be serialized and deserialized using
@@ -205,13 +253,13 @@ pub unsafe trait EnumSetType: Copy + Eq + EnumSetTypePrivate { }
 /// for serialization. This can be important for avoiding unintentional breaking changes when
 /// `EnumSet`s are serialized with formats like `bincode`.
 ///
-/// By default, unknown bits are ignored and silently removed from the bitset. To override this
+/// By default, unknown bits are ignored and silently removed from the bitset. To override thris
 /// behavior, you can add a `#[enumset(serialize_deny_unknown)]` attribute. This will cause
 /// deserialization to fail if an invalid bit is set.
 ///
 /// In addition, the `#[enumset(serialize_as_list)]` attribute causes the `EnumSet` to be
 /// instead serialized as a list of enum variants. This requires your enum type implement
-/// [`Serialize`] and [`Deserialize`]. Note that this is a breaking change
+/// [`Serialize`] and [`Deserialize`]. Note that this is a breaking change.
 #[derive(Copy, Clone, PartialEq, Eq)]
 #[repr(transparent)]
 pub struct EnumSet<T: EnumSetType> {
@@ -372,7 +420,8 @@ impl <T: EnumSetType> EnumSet<T> {
         self.__priv_repr = self.__priv_repr.and_not(other.__priv_repr);
     }
 
-    /// Creates an iterator over the values in this set.
+    /// Iterates the contents of the set in order from the least significant bit to the most
+    /// significant bit.
     ///
     /// Note that iterator invalidation is impossible as the iterator contains a copy of this type,
     /// rather than holding a reference to it.
@@ -397,7 +446,8 @@ macro_rules! conversion_impls {
             #[doc = "` representing the elements of this set.\n\nIf the underlying bitset will \
                      not fit in a `"]
             #[doc = $underlying_str]
-            #[doc = "`, this method will panic."]
+            #[doc = "` or contains bits that do not correspond to an enum variant, this method \
+                     will panic."]
             #[inline(always)]
             pub fn $to(&self) -> $underlying {
                 self.$try_to().expect("Bitset will not fit into this type.")
@@ -408,7 +458,8 @@ macro_rules! conversion_impls {
             #[doc = "` representing the elements of this set.\n\nIf the underlying bitset will \
                      not fit in a `"]
             #[doc = $underlying_str]
-            #[doc = "`, this method will instead return `None`."]
+            #[doc = "` or contains bits that do not correspond to an enum variant, this method \
+                     will instead return `None`."]
             #[inline(always)]
             pub fn $try_to(&self) -> Option<$underlying> {
                 EnumSetTypeRepr::$to_fn_opt(&self.__priv_repr)
@@ -419,7 +470,8 @@ macro_rules! conversion_impls {
             #[doc = "` representing the elements of this set.\n\nIf the underlying bitset will \
                      not fit in a `"]
             #[doc = $underlying_str]
-            #[doc = "`, this method will truncate any bits that don't fit."]
+            #[doc = "`, this method will truncate any bits that don't fit or do not correspond \
+                     to an enum variant."]
             #[inline(always)]
             pub fn $to_truncated(&self) -> $underlying {
                 EnumSetTypeRepr::$to_fn(&self.__priv_repr)
@@ -641,6 +693,7 @@ impl <T: EnumSetType> EnumSetIter<T> {
         EnumSetIter { set }
     }
 }
+
 impl <T: EnumSetType> Iterator for EnumSetIter<T> {
     type Item = T;
 
@@ -659,6 +712,18 @@ impl <T: EnumSetType> Iterator for EnumSetIter<T> {
     }
 }
 
+impl <T: EnumSetType> DoubleEndedIterator for EnumSetIter<T> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        if self.set.is_empty() {
+            None
+        } else {
+            let bit = T::Repr::WIDTH - 1 - self.set.__priv_repr.leading_zeros();
+            self.set.__priv_repr.remove_bit(bit);
+            unsafe { Some(T::enum_from_u32(bit)) }
+        }
+    }
+}
+
 impl<T: EnumSetType> ExactSizeIterator for EnumSetIter<T> {}
 
 impl<T: EnumSetType> Extend<T> for EnumSet<T> {
